@@ -70,6 +70,29 @@ pango_attr_type_register (const gchar *name)
   return current_type++;
 }
 
+/**
+ * pango_attribute_init:
+ * @attr: a #PangoAttribute
+ * @klass: a #PangoAttributeClass
+ *
+ * Initializes @attr's klass to @klass, it's start_index to zero,
+ * and end_index to %G_MAXUINT such that the attribute applies
+ * to the entire text by default.
+ *
+ * Since: 1.20
+ **/
+void
+pango_attribute_init (PangoAttribute       *attr,
+		      const PangoAttrClass *klass)
+{
+  g_return_if_fail (attr != NULL);
+  g_return_if_fail (klass != NULL);
+
+  attr->klass = klass;
+  attr->start_index = 0;
+  attr->end_index = G_MAXUINT;
+}
+
 /**
  * pango_attribute_copy:
  * @attr: a #PangoAttribute
@@ -158,8 +181,7 @@ pango_attr_string_new (const PangoAttrClass *klass,
 		       const char           *str)
 {
   PangoAttrString *result = g_slice_new (PangoAttrString);
-
-  result->attr.klass = klass;
+  pango_attribute_init (&result->attr, klass);
   result->value = g_strdup (str);
 
   return (PangoAttribute *)result;
@@ -231,11 +253,8 @@ pango_attr_language_new (PangoLanguage *language)
     pango_attr_language_equal
   };
 
-  g_return_val_if_fail (language != NULL, NULL);
-
   result = g_slice_new (PangoAttrLanguage);
-
-  result->attr.klass = &klass;
+  pango_attribute_init (&result->attr, &klass);
   result->value = language;
 
   return (PangoAttribute *)result;
@@ -279,7 +298,7 @@ pango_attr_color_new (const PangoAttrClass *klass,
 		      guint16               blue)
 {
   PangoAttrColor *result = g_slice_new (PangoAttrColor);
-  result->attr.klass = klass;
+  pango_attribute_init (&result->attr, klass);
   result->color.red = red;
   result->color.green = green;
   result->color.blue = blue;
@@ -370,7 +389,7 @@ pango_attr_int_new (const PangoAttrClass *klass,
 		    int                   value)
 {
   PangoAttrInt *result = g_slice_new (PangoAttrInt);
-  result->attr.klass = klass;
+  pango_attribute_init (&result->attr, klass);
   result->value = value;
 
   return (PangoAttribute *)result;
@@ -407,7 +426,7 @@ pango_attr_float_new  (const PangoAttrClass *klass,
 		       double                value)
 {
   PangoAttrFloat *result = g_slice_new (PangoAttrFloat);
-  result->attr.klass = klass;
+  pango_attribute_init (&result->attr, klass);
   result->value = value;
 
   return (PangoAttribute *)result;
@@ -462,7 +481,7 @@ pango_attr_size_new_internal (int size,
   };
 
   result = g_slice_new (PangoAttrSize);
-  result->attr.klass = absolute ? &absolute_klass : &klass;
+  pango_attribute_init (&result->attr, absolute ? &absolute_klass : &klass);
   result->size = size;
   result->absolute = absolute;
 
@@ -640,7 +659,7 @@ pango_attr_font_desc_new (const PangoFontDescription *desc)
   };
 
   PangoAttrFontDesc *result = g_slice_new (PangoAttrFontDesc);
-  result->attr.klass = &klass;
+  pango_attribute_init (&result->attr, &klass);
   result->desc = pango_font_description_copy (desc);
 
   return (PangoAttribute *)result;
@@ -936,7 +955,7 @@ pango_attr_shape_new_with_data (const PangoRectangle  *ink_rect,
   g_return_val_if_fail (logical_rect != NULL, NULL);
 
   result = g_slice_new (PangoAttrShape);
-  result->attr.klass = &klass;
+  pango_attribute_init (&result->attr, &klass);
   result->ink_rect = *ink_rect;
   result->logical_rect = *logical_rect;
   result->data = data;
@@ -1060,7 +1079,7 @@ pango_attr_list_new (void)
 
 /**
  * pango_attr_list_ref:
- * @list: a #PangoAttrList
+ * @list: a #PangoAttrList, may be %NULL
  *
  * Increase the reference count of the given attribute list by one.
  *
@@ -1071,7 +1090,8 @@ pango_attr_list_new (void)
 PangoAttrList *
 pango_attr_list_ref (PangoAttrList *list)
 {
-  g_return_val_if_fail (list != NULL, NULL);
+  if (list == NULL)
+    return NULL;
 
   list->ref_count++;
 
@@ -1080,7 +1100,7 @@ pango_attr_list_ref (PangoAttrList *list)
 
 /**
  * pango_attr_list_unref:
- * @list: a #PangoAttrList
+ * @list: a #PangoAttrList, may be %NULL
  *
  * Decrease the reference count of the given attribute list by one.
  * If the result is zero, free the attribute list and the attributes
@@ -1091,7 +1111,9 @@ pango_attr_list_unref (PangoAttrList *list)
 {
   GSList *tmp_list;
 
-  g_return_if_fail (list != NULL);
+  if (list == NULL)
+    return;
+
   g_return_if_fail (list->ref_count > 0);
 
   list->ref_count--;
@@ -1114,13 +1136,14 @@ pango_attr_list_unref (PangoAttrList *list)
 
 /**
  * pango_attr_list_copy:
- * @list: a #PangoAttrList
+ * @list: a #PangoAttrList, may be %NULL
  *
  * Copy @list and return an identical new list.
  *
  * Return value: the newly allocated #PangoAttrList, with a
  *               reference count of one, which should
  *               be freed with pango_attr_list_unref().
+ *               Returns %NULL if @list was %NULL.
  **/
 PangoAttrList *
 pango_attr_list_copy (PangoAttrList *list)
@@ -1129,7 +1152,8 @@ pango_attr_list_copy (PangoAttrList *list)
   GSList *iter;
   GSList *new_attrs;
 
-  g_return_val_if_fail (list != NULL, NULL);
+  if (list == NULL)
+    return NULL;
 
   new = pango_attr_list_new ();
 
@@ -1166,7 +1190,7 @@ pango_attr_list_insert_internal (PangoAttrList  *list,
   else if (((PangoAttribute *)list->attributes_tail->data)->start_index < start_index ||
 	   (!before && ((PangoAttribute *)list->attributes_tail->data)->start_index == start_index))
     {
-      g_slist_append (list->attributes_tail, attr);
+      list->attributes_tail = g_slist_append (list->attributes_tail, attr);
       list->attributes_tail = list->attributes_tail->next;
       g_assert (list->attributes_tail);
     }
