@@ -53,6 +53,14 @@ struct _PangoWin32SizeInfo
   GSList *logfontws;
 };
 
+#if !defined(NTM_PS_OPENTYPE)
+# define NTM_PS_OPENTYPE 0x20000
+#endif
+
+#if !defined(NTM_TYPE1)
+# define NTM_TYPE1 0x100000
+#endif
+
 #define PANGO_WIN32_TYPE_FAMILY              (pango_win32_family_get_type ())
 #define PANGO_WIN32_FAMILY(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), PANGO_WIN32_TYPE_FAMILY, PangoWin32Family))
 #define PANGO_WIN32_IS_FAMILY(object)        (G_TYPE_CHECK_INSTANCE_TYPE ((object), PANGO_WIN32_TYPE_FAMILY))
@@ -181,23 +189,25 @@ pango_win32_inner_enum_proc (LOGFONTW    *lfp,
 }
 
 static int CALLBACK
-pango_win32_enum_proc (LOGFONTW    *lfp,
-		       TEXTMETRICW *metrics,
-		       DWORD        fontType,
-		       LPARAM       lParam)
+pango_win32_enum_proc (LOGFONTW       *lfp,
+		       NEWTEXTMETRICW *metrics,
+		       DWORD           fontType,
+		       LPARAM          lParam)
 {
   LOGFONTW lf;
 
-  PING(("%S", lfp->lfFaceName));
+  PING(("%S: %lu %lx", lfp->lfFaceName, fontType, metrics->ntmFlags));
 
-  if (fontType != TRUETYPE_FONTTYPE)
-    return 1;
+  if (fontType == TRUETYPE_FONTTYPE ||
+      (_pango_win32_os_version_info.dwMajorVersion >= 5 &&
+       ((metrics->ntmFlags & NTM_PS_OPENTYPE) || (metrics->ntmFlags & NTM_TYPE1))))
+    {
+      lf = *lfp;
 
-  lf = *lfp;
-
-  EnumFontFamiliesExW (_pango_win32_hdc, &lf,
-		       (FONTENUMPROCW) pango_win32_inner_enum_proc,
-		       lParam, 0);
+      EnumFontFamiliesExW (_pango_win32_hdc, &lf,
+			   (FONTENUMPROCW) pango_win32_inner_enum_proc,
+			   lParam, 0);
+    }
 
   return 1;
 }
@@ -786,6 +796,7 @@ pango_win32_font_description_from_logfont (const LOGFONT *lfp)
 
   description = pango_font_description_new ();
   pango_font_description_set_family (description, family);
+  g_free(family);
   pango_font_description_set_style (description, style);
   pango_font_description_set_weight (description, weight);
   pango_font_description_set_stretch (description, stretch);
@@ -981,6 +992,7 @@ pango_win32_font_description_from_logfontw (const LOGFONTW *lfp)
 
   description = pango_font_description_new ();
   pango_font_description_set_family (description, family);
+  g_free(family);
   pango_font_description_set_style (description, style);
   pango_font_description_set_weight (description, weight);
   pango_font_description_set_stretch (description, stretch);
@@ -1090,6 +1102,7 @@ pango_win32_insert_font (PangoWin32FontMap *win32fontmap,
 
   win32face->is_synthetic = is_synthetic;
 
+  win32face->has_cmap = TRUE;
   win32face->cmap_format = 0;
   win32face->cmap = NULL;
 
